@@ -942,6 +942,16 @@ content_length_in_encoding(Enc, Stream, Bytes) :-
 %     * html(+Tokens)
 %     Result of html//1 from html_write.pl
 %
+%     * json(+Term)
+%     Posting a JSON query and processing the JSON reply (or any other
+%     reply understood by http_read_data/3) is simple as
+%     =|http_post(URL, json(Term), Reply, [])|=, where Term is a JSON
+%     term as described in json.pl and reply is of the same format if
+%     the server replies with JSON, when using module =|:-
+%     use_module(library(http/http_json))|=. Note that the module is
+%     used in both http server and http client, see
+%     library(http/http_json).
+%
 %     * xml(+Term)
 %     Post the result of xml_write/3 using the Mime-type
 %     =|text/xml|=
