@@ -43,6 +43,13 @@
 #include <assert.h>
 #endif
 
+#ifdef _MSC_VER
+static inline size_t
+__builtin_popcountll(long long sz)
+{ return __popcnt64(sz);
+}
+#endif
+
 typedef void *(*mp_malloc_t)(size_t);
 typedef void *(*mp_realloc_t)(void *, size_t old, size_t newsize);
 typedef void  (*mp_free_t)(void *, size_t size);
@@ -50,9 +57,11 @@ typedef void  (*mp_free_t)(void *, size_t size);
 typedef struct mp_alloc_wrapper
 { bf_context_t bf_context;
   mp_realloc_t realloc_func;
+  mp_free_t free_func;
 } mp_alloc_wrapper;
 
 extern mp_alloc_wrapper alloc_wrapper;
+
 extern void bf_not_implemented(const char *func);
 
 void	bf_print_i(const char *msg, const bf_t *i);
@@ -64,14 +73,15 @@ void	bf_print_i(const char *msg, const bf_t *i);
 
 static inline void
 mp_get_memory_functions(mp_malloc_t *m, mp_realloc_t *r, mp_free_t *f)
-{ *m = NULL;
-  *r = alloc_wrapper.realloc_func;
-  *f = NULL;
+{ if(m) *m = NULL;
+  if(r) *r = alloc_wrapper.realloc_func;
+  if(f) *f = alloc_wrapper.free_func;
 }
 
 static inline void
 mp_set_memory_functions(mp_malloc_t m, mp_realloc_t r, mp_free_t f)
 { alloc_wrapper.realloc_func = r;
+  alloc_wrapper.free_func = f;
 }
 
 		 /*******************************
@@ -161,7 +171,7 @@ mpz_get_si(const mpz_t n)
 { int64_t nv;
 
   if ( bf_get_int64(&nv, n, BF_RNDN) == 0 )
-    return nv;
+    return (long)nv;
 
   assert(0);				/* TBD: return least significant bits */
   return 0;
@@ -245,7 +255,7 @@ mpz_popcount(const mpz_t n)
 { mp_bitcnt_t cnt = 0;
 
   for(size_t i=0; i<n->len; i++)
-    cnt += __builtin_popcountll(n->tab[i]);
+    cnt += (mp_bitcnt_t)__builtin_popcountll(n->tab[i]);
 
   return cnt;
 }
@@ -577,7 +587,7 @@ mpz_tdiv_q_ui(mpz_t Q, const mpz_t N, unsigned long d)
   bf_divrem(Q, &rem, N, D, BF_PREC_INF, 0, BF_RNDZ);
   bf_get_int64(&r, &rem, BF_RNDN);
   bf_delete(&rem);
-  return r;
+  return (unsigned long)r;
 }
 
 
@@ -615,24 +625,7 @@ mpz_root(mpz_t ROP, const mpz_t OP, unsigned long int N)
 void	mpz_pow_ui(mpz_t r, const mpz_t x, unsigned long y);
 void	mpz_ui_pow_ui(mpz_t r, unsigned long x, unsigned long y);
 void	mpz_powm(mpz_t r, const mpz_t base, const mpz_t exp, const mpz_t mod);
-
-static inline char *
-mpz_get_str(char *STR, int BASE, const mpz_t OP)
-{ const bf_t *op = OP;
-  bf_t copy;
-
-  if ( !op->ctx )
-  { copy = OP[0];
-    copy.ctx = &alloc_wrapper.bf_context;
-    op = &copy;
-  }
-
-  char *s = bf_ftoa(NULL, op, BASE, 0, BF_RNDZ|BF_FTOA_FORMAT_FRAC);
-  strcpy(STR, s);
-  bf_free(op->ctx, s);
-
-  return STR;
-}
+char   *mpz_get_str(char *STR, int BASE, const mpz_t OP);
 
 
 		 /*******************************
