@@ -167,15 +167,15 @@ query_loop(atom_t goal, int loop)
     if ( fid ) PL_discard_foreign_frame(fid);
     if ( !except )
       break;
-#ifdef O_PLMT
-    if (LD->exit_requested)
+#ifdef O_ENGINES
+    if (LD->thread.exit_requested)
       loop = 0;
 #endif
   } while(loop);
 
 #ifdef O_PLMT
   DEBUG(MSG_CLEANUP_THREAD,
-	if ( LD->exit_requested )
+	if ( LD->thread.exit_requested )
 	Sdprintf("Thread %d: leaving REPL loop due to exit_requested\n",
 		 PL_thread_self()));
 #endif
@@ -684,17 +684,18 @@ check_data(DECL_LD Word p, chk_data *context)
 
 last_arg:
 
+#ifndef O_ATTVAR
+#define isAttVar(p) FALSE
+#endif
+
   while(isRef(*p))
   { assert(!is_marked(p));
     p2 = unRef(*p);
     DEBUG(CHK_HIGHER_ADDRESS,
 	  { if ( p2 > p )
-	    {
-#ifdef O_ATTVAR
-	      if ( !isAttVar(*p2) )
-#endif
-		if ( !gc_status.blocked )
-		  printk(context, "Reference to higher address");
+	    { if ( !isAttVar(*p2) &&
+		   !gc_status.blocked )
+		printk(context, "Reference to higher address");
 	    }
 	  });
     if ( p2 == p )
